@@ -309,6 +309,9 @@ CMARK_EXPORT
 bool cmark_node_set_user_data_free_func(cmark_node *node,
                                         CMarkNodeUserDataFreeFunc free_func);
 
+CMARK_EXPORT
+void cmark_node_set_html_attrs(cmark_node *node, const char *attrs);
+
 /** Returns the type of 'node', or `CMARK_NODE_NONE` on error.
  */
 CMARK_EXPORT cmark_node_type cmark_node_get_type(cmark_node *node);
@@ -580,6 +583,9 @@ cmark_node *cmark_parser_finish(cmark_parser *parser);
 CMARK_EXPORT
 bool cmark_parser_attach_syntax_extension(cmark_parser *parser, cmark_syntax_extension *extension);
 
+CMARK_EXPORT
+void cmark_parser_add_reference(cmark_parser *parser, const char *label, const char *url, const char *title);
+
 /** Parse a CommonMark document in 'buffer' of length 'len'.
  * Returns a pointer to a tree of nodes.
  */
@@ -597,6 +603,9 @@ cmark_node *cmark_parse_file(FILE *f, int options);
 CMARK_EXPORT
 int cmark_parser_get_line_number(cmark_parser *parser);
 
+CMARK_EXPORT
+cmark_node *cmark_parser_get_root(cmark_parser *parser);
+
 /** Return the offset in bytes in the line being processed.
  *
  * Example:
@@ -970,7 +979,7 @@ cmark_syntax_extension *cmark_syntax_extension_new (const char *name);
 
 /** Should return 'true' if the predicate matches 'c', 'false' otherwise
  */
-typedef int (*CMarkInlinePredicate)(int c);
+typedef int (*CMarkInlinePredicate)(int c, int pos, void *user_data);
 
 /** Advance the current inline parsing offset */
 CMARK_EXPORT
@@ -999,7 +1008,7 @@ bool cmark_inline_parser_is_eof(cmark_inline_parser *parser);
  * while 'pred' matches. Free after usage.
  */
 CMARK_EXPORT
-char *cmark_inline_parser_take_while(cmark_inline_parser *parser, CMarkInlinePredicate pred);
+char *cmark_inline_parser_take_while(cmark_inline_parser *parser, CMarkInlinePredicate pred, void *user_data);
 
 /** Push a delimiter on the delimiter stack.
  * See <<http://spec.commonmark.org/0.24/#phase-2-inline-structure> for
