@@ -1259,12 +1259,12 @@ my_strndup (const char *s, size_t n)
   return (char *) memcpy (result, s, len);
 }
 
-char *cmark_inline_parser_take_while(cmark_inline_parser *parser, CMarkInlinePredicate pred) {
+char *cmark_inline_parser_take_while(cmark_inline_parser *parser, CMarkInlinePredicate pred, void *user_data) {
   unsigned char c;
   bufsize_t startpos = parser->pos;
   bufsize_t len = 0;
 
-  while ((c = peek_char(parser)) && (*pred)(c)) {
+  while ((c = peek_char(parser)) && (*pred)(c, parser->pos, user_data)) {
     advance(parser);
     len++;
   }
