@@ -73,6 +73,7 @@ static bool S_can_contain(cmark_node *node, cmark_node *child) {
            child->type == CMARK_NODE_STRONG ||
            child->type == CMARK_NODE_LINK ||
            child->type == CMARK_NODE_IMAGE ||
+           child->type == CMARK_NODE_HTML_INLINE ||
            child->type == CMARK_NODE_STRIKETHROUGH;
 
   default:
@@ -152,6 +153,8 @@ static void S_free_nodes(cmark_node *e) {
       e->next = e->first_child;
     }
     next = e->next;
+    if (e->html_attrs)
+      free (e->html_attrs);
     free(e);
     e = next;
   }
@@ -309,6 +312,13 @@ bool cmark_node_set_user_data_free_func(cmark_node *node,
   return 1;
 }
 
+void cmark_node_set_html_attrs(cmark_node *node, const char *attrs)
+{
+  if (node->html_attrs)
+    free (node->html_attrs);
+  node->html_attrs = strdup (attrs);
+}
+
 const char *cmark_node_get_literal(cmark_node *node) {
   if (node == NULL) {
     return NULL;
