@@ -21,34 +21,32 @@
 # pylint: disable=no-self-use
 # pylint: disable=too-few-public-methods
 # pylint: disable=too-many-instance-attributes
-
 import unittest
 import shutil
 import io
 import os
 
-from hotdoc.core.change_tracker import ChangeTracker
-from hotdoc.core.doc_database import DocDatabase
-from hotdoc.core.doc_repo import CoreExtension
+from hotdoc.core.filesystem import ChangeTracker
+from hotdoc.core.database import Database
+from hotdoc.core.project import CoreExtension
 from hotdoc.core.symbols import FunctionSymbol
 from hotdoc.core.links import LinkResolver
-from hotdoc.parsers.standalone_parser import SitemapParser
+from hotdoc.parsers.sitemap import SitemapParser
 from hotdoc.parsers import cmark
-from hotdoc.core.doc_tree import DocTree
-from hotdoc.core.base_extension import BaseExtension
+from hotdoc.core.tree import Tree
+from hotdoc.core.extension import Extension
 from hotdoc.utils.utils import OrderedSet, touch
+from hotdoc.core.config import Config
 
 
-class TestExtension(BaseExtension):
+class TestExtension(Extension):
     extension_name = 'test-extension'
     argument_prefix = 'test'
 
-    def __init__(self, doc_repo):
-        self.formatters = {'html': None}
-        self.smart_index = True
-        super(TestExtension, self).__init__(doc_repo)
-
-    def setup(self):
+    # pylint: disable=arguments-differ
+    def setup(self, smart=True):
+        super(TestExtension, self).setup()
+        self.smart_index = smart
         stale, _ = self.get_stale_files(self.sources)
         for source in stale:
             with open(source, 'r') as _:
@@ -62,12 +60,12 @@ class TestExtension(BaseExtension):
         return self.sources
 
 
-class TestDocTree(unittest.TestCase):
+class TestTree(unittest.TestCase):
     def setUp(self):
         here = os.path.dirname(__file__)
         self.__md_dir = os.path.abspath(os.path.join(
             here, 'tmp-markdown-files'))
-        self.__priv_dir = os.path.abspath(os.path.join(
+        self.private_folder = os.path.abspath(os.path.join(
             here, 'tmp-private'))
         self.__src_dir = os.path.abspath(os.path.join(
             here, 'tmp-src-files'))
@@ -75,24 +73,40 @@ class TestDocTree(unittest.TestCase):
             here, 'tmp-output'))
         self.__remove_tmp_dirs()
         os.mkdir(self.__md_dir)
-        os.mkdir(self.__priv_dir)
+        os.mkdir(self.private_folder)
         os.mkdir(self.__src_dir)
         os.mkdir(self.get_generated_doc_folder())
         self.include_paths = OrderedSet([self.__md_dir])
         self.include_paths.add(self.get_generated_doc_folder())
 
+        self.dependency_map = {}
+
         # Using the real doc database is too costly, tests should be lightning
         # fast (and they are)
-        self.doc_database = DocDatabase()
-        self.doc_database.setup(self.__priv_dir)
-        self.link_resolver = LinkResolver(self.doc_database)
+        self.database = Database(self.private_folder)
+        self.link_resolver = LinkResolver(self.database)
 
         self.change_tracker = ChangeTracker()
 
         self.sitemap_parser = SitemapParser()
 
-        self.test_ext = TestExtension(self)
-        self.core_ext = CoreExtension(self)
+        self.project_name = 'test-project'
+        self.sanitized_name = 'test-project-0.1'
+        self.incremental = False
+
+        self.tree = Tree(self, self)
+
+        self.test_ext = TestExtension(self, self)
+        self.core_ext = CoreExtension(self, self)
+
+        cfg = Config()
+
+        self.test_ext.parse_toplevel_config(cfg)
+        self.test_ext.parse_config(cfg)
+        self.core_ext.parse_toplevel_config(cfg)
+        self.core_ext.parse_config(cfg)
+        self.subprojects = {}
+        self.is_toplevel = True
 
     def tearDown(self):
         self.__remove_tmp_dirs()
@@ -100,13 +114,13 @@ class TestDocTree(unittest.TestCase):
         del self.core_ext
 
     def get_generated_doc_folder(self):
-        return os.path.join(self.__priv_dir, 'generated')
+        return os.path.join(self.private_folder, 'generated')
 
     def get_base_doc_folder(self):
         return self.__md_dir
 
     def get_private_folder(self):
-        return self.__priv_dir
+        return self.private_folder
 
     def __parse_sitemap(self, text):
         path = os.path.join(self.__md_dir, 'sitemap.txt')
@@ -153,7 +167,7 @@ class TestDocTree(unittest.TestCase):
 
     def __remove_tmp_dirs(self):
         shutil.rmtree(self.__md_dir, ignore_errors=True)
-        shutil.rmtree(self.__priv_dir, ignore_errors=True)
+        shutil.rmtree(self.private_folder, ignore_errors=True)
         shutil.rmtree(self.__src_dir, ignore_errors=True)
         shutil.rmtree(self.__output_dir, ignore_errors=True)
         shutil.rmtree(self.get_generated_doc_folder(), ignore_errors=True)
@@ -169,10 +183,9 @@ class TestDocTree(unittest.TestCase):
             'section.markdown',
             (u'# My section\n'))
 
-        doc_tree = DocTree(self.__priv_dir, self.include_paths)
-        doc_tree.parse_sitemap(self.change_tracker, sitemap)
+        self.tree.parse_sitemap(sitemap)
 
-        pages = doc_tree.get_pages()
+        pages = self.tree.get_pages()
 
         # We do not care about ordering
         self.assertSetEqual(
@@ -194,61 +207,64 @@ class TestDocTree(unittest.TestCase):
             'section.markdown',
             (u'# My section\n'))
 
-        doc_tree = DocTree(self.__priv_dir, self.include_paths)
-        doc_tree.parse_sitemap(self.change_tracker, sitemap)
+        self.tree.parse_sitemap(sitemap)
 
         # Building from scratch, all pages are stale
         self.assertSetEqual(
-            set(doc_tree.get_stale_pages()),
+            set(self.tree.get_stale_pages()),
             set([u'index.markdown',
                  u'section.markdown']))
 
-        doc_tree.persist()
+        self.tree.persist()
 
-        doc_tree = DocTree(self.__priv_dir, self.include_paths)
-        doc_tree.parse_sitemap(self.change_tracker, sitemap)
+        self.incremental = True
+        self.tree = Tree(self, self)
+        self.tree.parse_sitemap(sitemap)
 
         # Nothing changed, no page is stale
         self.assertSetEqual(
-            set(doc_tree.get_stale_pages()),
+            set(self.tree.get_stale_pages()),
             set({}))
 
         # But we still have our pages
         self.assertSetEqual(
-            set(doc_tree.get_pages()),
+            set(self.tree.get_pages()),
             set([u'index.markdown',
                  u'section.markdown']))
 
         touch(os.path.join(self.__md_dir, u'section.markdown'))
 
-        doc_tree = DocTree(self.__priv_dir, self.include_paths)
-        doc_tree.parse_sitemap(self.change_tracker, sitemap)
+        self.tree = Tree(self, self)
+        self.tree.parse_sitemap(sitemap)
 
         self.assertSetEqual(
-            set(doc_tree.get_stale_pages()),
+            set(self.tree.get_stale_pages()),
             set([u'section.markdown']))
 
-    def __assert_extension_names(self, doc_tree, name_map):
-        pages = doc_tree.get_pages()
-        for name, ext_name in name_map.items():
+    def __assert_extension_names(self, tree, name_map):
+        pages = tree.get_pages()
+        for name, ext_name in list(name_map.items()):
             page = pages[name]
             self.assertEqual(ext_name, page.extension_name)
 
-    def __assert_stale(self, doc_tree, expected_stale):
-        stale_pages = doc_tree.get_stale_pages()
+    def __assert_stale(self, expected_stale):
+        stale_pages = self.tree.get_stale_pages()
         for pagename in expected_stale:
             self.assertIn(pagename, stale_pages)
             stale_pages.pop(pagename)
         self.assertEqual(len(stale_pages), 0)
 
-    def __create_test_layout(self):
-        inp = (u'index.markdown\n'
-               '\ttest-index\n'
-               '\t\ttest-section.markdown\n'
-               '\t\t\tsource_a.test\n'
-               '\t\tpage_x.markdown\n'
-               '\t\tpage_y.markdown\n'
-               '\tcore_page.markdown\n')
+    def __create_test_layout(self, with_ext_index=True, sitemap=None):
+        if not sitemap:
+            inp = (u'index.markdown\n'
+                   '\ttest-index\n'
+                   '\t\ttest-section.markdown\n'
+                   '\t\t\tsource_a.test\n'
+                   '\t\tpage_x.markdown\n'
+                   '\t\tpage_y.markdown\n'
+                   '\tcore_page.markdown\n')
+        else:
+            inp = sitemap
 
         sources = []
 
@@ -262,7 +278,9 @@ class TestDocTree(unittest.TestCase):
             ['symbol_3',
              'symbol_4']))
 
-        self.test_ext.index = 'test-index.markdown'
+        if with_ext_index:
+            self.test_ext.index = 'test-index.markdown'
+
         self.test_ext.sources = sources
         self.test_ext.setup()
 
@@ -274,9 +292,10 @@ class TestDocTree(unittest.TestCase):
         self.__create_md_file(
             'core_page.markdown',
             (u'# My non-extension page\n'))
-        self.__create_md_file(
-            'test-index.markdown',
-            (u'# My test index\n'))
+        if with_ext_index:
+            self.__create_md_file(
+                'test-index.markdown',
+                (u'# My test index\n'))
         self.__create_md_file(
             'test-section.markdown',
             (u'# My test section\n'
@@ -292,23 +311,20 @@ class TestDocTree(unittest.TestCase):
             'page_y.markdown',
             (u'# Page Y\n'))
 
-        doc_tree = DocTree(self.__priv_dir, self.include_paths)
-        doc_tree.parse_sitemap(self.change_tracker, sitemap)
+        self.tree.parse_sitemap(sitemap)
 
-        return doc_tree, sitemap
+        return sitemap
 
-    def __update_test_layout(self, doc_tree, sitemap):
+    def __update_test_layout(self, sitemap):
         self.test_ext.reset()
+        self.tree = Tree(self, self)
         self.test_ext.setup()
-
-        doc_tree = DocTree(self.__priv_dir, self.include_paths)
-        doc_tree.parse_sitemap(self.change_tracker, sitemap)
-        return doc_tree
+        self.tree.parse_sitemap(sitemap)
 
     def test_extension_basic(self):
-        doc_tree, _ = self.__create_test_layout()
+        _ = self.__create_test_layout()
         self.__assert_extension_names(
-            doc_tree,
+            self.tree,
             {u'index.markdown': 'core',
              u'test-index': 'test-extension',
              u'test-section.markdown': 'test-extension',
@@ -318,9 +334,9 @@ class TestDocTree(unittest.TestCase):
              u'page_y.markdown': 'test-extension',
              u'core_page.markdown': 'core'})
 
-        all_pages = doc_tree.get_pages()
+        all_pages = self.tree.get_pages()
         self.assertEqual(len(all_pages), 8)
-        self.__assert_stale(doc_tree, all_pages)
+        self.__assert_stale(all_pages)
         self.assertNotIn('source_a.test', all_pages['test-index'].subpages)
         self.assertIn('source_a.test',
                       all_pages['test-section.markdown'].subpages)
@@ -329,8 +345,8 @@ class TestDocTree(unittest.TestCase):
         self.__create_md_file(
             'source_a.test.markdown',
             (u'# My override\n'))
-        doc_tree, _ = self.__create_test_layout()
-        page = doc_tree.get_pages()['source_a.test']
+        _ = self.__create_test_layout()
+        page = self.tree.get_pages()['source_a.test']
 
         self.assertEqual(
             page.symbol_names,
@@ -341,10 +357,18 @@ class TestDocTree(unittest.TestCase):
             os.path.basename(page.source_file),
             'source_a.test.markdown')
 
+        out, _ = cmark.ast_to_html(page.ast, None)
+
         self.assertEqual(
-            cmark.ast_to_html(page.ast, None),
+            out,
             u'<h1>My override</h1>\n')
 
+    def test_no_extension_index_override(self):
+        _ = self.__create_test_layout(with_ext_index=False)
+        ext_index = self.tree.get_pages()['test-index']
+        self.assertEqual(ext_index.generated, True)
+        self.assertEqual(len(ext_index.subpages), 4)
+
     def test_parse_yaml(self):
         inp = (u'index.markdown\n')
         sitemap = self.__parse_sitemap(inp)
@@ -356,13 +380,15 @@ class TestDocTree(unittest.TestCase):
              '...\n'
              '# My documentation\n'))
 
-        doc_tree = DocTree(self.__priv_dir, self.include_paths)
-        doc_tree.parse_sitemap(self.change_tracker, sitemap)
+        self.tree.parse_sitemap(sitemap)
 
-        pages = doc_tree.get_pages()
+        pages = self.tree.get_pages()
         page = pages.get('index.markdown')
+
+        out, _ = cmark.ast_to_html(page.ast, None)
+
         self.assertEqual(
-            cmark.ast_to_html(page.ast, None),
+            out,
             u'<h1>My documentation</h1>\n')
 
         self.assertEqual(page.title, u'A random title')
@@ -385,14 +411,13 @@ class TestDocTree(unittest.TestCase):
              '\n'
              '[](index.markdown)\n'))
 
-        doc_tree = DocTree(self.__priv_dir, self.include_paths)
-        doc_tree.parse_sitemap(self.change_tracker, sitemap)
-        doc_tree.resolve_symbols(self.doc_database, self.link_resolver)
-        doc_tree.format(
+        self.tree.parse_sitemap(sitemap)
+        self.tree.resolve_symbols(self.database, self.link_resolver)
+        self.tree.format(
             self.link_resolver, self.__output_dir,
             {self.core_ext.extension_name: self.core_ext})
 
-        pages = doc_tree.get_pages()
+        pages = self.tree.get_pages()
         page = pages.get('section.markdown')
         self.assertEqual(
             page.formatted_contents,
@@ -412,14 +437,13 @@ class TestDocTree(unittest.TestCase):
              '\n'
              '[a label](index.markdown)\n'))
 
-        doc_tree = DocTree(self.__priv_dir, self.include_paths)
-        doc_tree.parse_sitemap(self.change_tracker, sitemap)
-        doc_tree.resolve_symbols(self.doc_database, self.link_resolver)
-        doc_tree.format(
+        self.tree.parse_sitemap(sitemap)
+        self.tree.resolve_symbols(self.database, self.link_resolver)
+        self.tree.format(
             self.link_resolver, self.__output_dir,
             {self.core_ext.extension_name: self.core_ext})
 
-        pages = doc_tree.get_pages()
+        pages = self.tree.get_pages()
         page = pages.get('section.markdown')
         self.assertEqual(
             page.formatted_contents,
@@ -439,14 +463,13 @@ class TestDocTree(unittest.TestCase):
              '\n'
              '[](index.markdown#subsection)\n'))
 
-        doc_tree = DocTree(self.__priv_dir, self.include_paths)
-        doc_tree.parse_sitemap(self.change_tracker, sitemap)
-        doc_tree.resolve_symbols(self.doc_database, self.link_resolver)
-        doc_tree.format(
+        self.tree.parse_sitemap(sitemap)
+        self.tree.resolve_symbols(self.database, self.link_resolver)
+        self.tree.format(
             self.link_resolver, self.__output_dir,
             {self.core_ext.extension_name: self.core_ext})
 
-        pages = doc_tree.get_pages()
+        pages = self.tree.get_pages()
         page = pages.get('section.markdown')
         self.assertEqual(
             page.formatted_contents,
@@ -455,26 +478,27 @@ class TestDocTree(unittest.TestCase):
 
     # pylint: disable=too-many-statements
     def test_extension_incremental(self):
-        doc_tree, sitemap = self.__create_test_layout()
-        doc_tree.persist()
+        sitemap = self.__create_test_layout()
+        self.tree.persist()
+
+        self.incremental = True
 
         # Here we touch source_a.test, as its symbols were
         # all contained in a generated page, only that page
         # should now be stale
         self.__touch_src_file('source_a.test')
-        doc_tree = self.__update_test_layout(doc_tree, sitemap)
-        self.__assert_stale(doc_tree, set(['source_a.test']))
-        doc_tree.persist()
+        self.__update_test_layout(sitemap)
+        self.__assert_stale(set(['source_a.test']))
+        self.tree.persist()
 
         # We now touch source_b.test, which symbols are contained
         # both in a generated page and a user-provided one.
         # We expect both pages to be stale
         self.__touch_src_file('source_b.test')
-        doc_tree = self.__update_test_layout(doc_tree, sitemap)
-        self.__assert_stale(doc_tree,
-                            set(['source_b.test',
+        self.__update_test_layout(sitemap)
+        self.__assert_stale(set(['source_b.test',
                                  'page_x.markdown']))
-        doc_tree.persist()
+        self.tree.persist()
 
         # This one is trickier: we unlist symbol_3 from
         # page_x, which means the symbol should now be
@@ -484,20 +508,19 @@ class TestDocTree(unittest.TestCase):
         self.__create_md_file(
             'page_x.markdown',
             (u'# Page X\n'))
-        doc_tree = self.__update_test_layout(doc_tree, sitemap)
-        self.__assert_stale(doc_tree,
-                            set(['source_b.test',
+        self.__update_test_layout(sitemap)
+        self.__assert_stale(set(['source_b.test',
                                  'page_x.markdown']))
 
-        page_x = doc_tree.get_pages()['page_x.markdown']
+        page_x = self.tree.get_pages()['page_x.markdown']
         self.assertEqual(page_x.symbol_names, OrderedSet())
 
-        source_b_page = doc_tree.get_pages()['source_b.test']
+        source_b_page = self.tree.get_pages()['source_b.test']
         self.assertEqual(
             source_b_page.symbol_names,
             OrderedSet(['symbol_4', 'symbol_3']))
 
-        doc_tree.persist()
+        self.tree.persist()
 
         # Let's make sure the opposite use case works as well,
         # we relocate symbol_3 in page_x , both page_x and
@@ -511,20 +534,19 @@ class TestDocTree(unittest.TestCase):
              '...\n'
              '# Page X\n'))
 
-        doc_tree = self.__update_test_layout(doc_tree, sitemap)
-        self.__assert_stale(doc_tree,
-                            set(['source_b.test',
+        self.__update_test_layout(sitemap)
+        self.__assert_stale(set(['source_b.test',
                                  'page_x.markdown']))
 
-        page_x = doc_tree.get_pages()['page_x.markdown']
+        page_x = self.tree.get_pages()['page_x.markdown']
         self.assertEqual(page_x.symbol_names, OrderedSet(['symbol_3']))
 
-        source_b_page = doc_tree.get_pages()['source_b.test']
+        source_b_page = self.tree.get_pages()['source_b.test']
         self.assertEqual(
             source_b_page.symbol_names,
             OrderedSet(['symbol_4']))
 
-        doc_tree.persist()
+        self.tree.persist()
 
         # We now move the definition of symbol_3 to source_a.test,
         # we thus expect the generated page for source_a.test to be
@@ -540,28 +562,27 @@ class TestDocTree(unittest.TestCase):
         self.__create_src_file(
             'source_b.test',
             ['symbol_4'])
-        doc_tree = self.__update_test_layout(doc_tree, sitemap)
+        self.__update_test_layout(sitemap)
 
-        self.__assert_stale(doc_tree,
-                            set(['source_a.test',
+        self.__assert_stale(set(['source_a.test',
                                  'source_b.test',
                                  'page_x.markdown']))
 
-        page_x = doc_tree.get_pages()['page_x.markdown']
+        page_x = self.tree.get_pages()['page_x.markdown']
         self.assertEqual(page_x.symbol_names, OrderedSet(['symbol_3']))
 
-        source_b_page = doc_tree.get_pages()['source_b.test']
+        source_b_page = self.tree.get_pages()['source_b.test']
         self.assertEqual(
             source_b_page.symbol_names,
             OrderedSet(['symbol_4']))
 
-        source_a_page = doc_tree.get_pages()['source_a.test']
+        source_a_page = self.tree.get_pages()['source_a.test']
         self.assertEqual(
             source_a_page.symbol_names,
             OrderedSet(['symbol_1',
                         'symbol_2']))
 
-        doc_tree.persist()
+        self.tree.persist()
 
         # And we rollback again
         self.__create_src_file(
@@ -572,28 +593,27 @@ class TestDocTree(unittest.TestCase):
             'source_b.test',
             ['symbol_3',
              'symbol_4'])
-        doc_tree = self.__update_test_layout(doc_tree, sitemap)
+        self.__update_test_layout(sitemap)
 
-        self.__assert_stale(doc_tree,
-                            set(['source_a.test',
+        self.__assert_stale(set(['source_a.test',
                                  'source_b.test',
                                  'page_x.markdown']))
 
-        page_x = doc_tree.get_pages()['page_x.markdown']
+        page_x = self.tree.get_pages()['page_x.markdown']
         self.assertEqual(page_x.symbol_names, OrderedSet(['symbol_3']))
 
-        source_b_page = doc_tree.get_pages()['source_b.test']
+        source_b_page = self.tree.get_pages()['source_b.test']
         self.assertEqual(
             source_b_page.symbol_names,
             OrderedSet(['symbol_4']))
 
-        source_a_page = doc_tree.get_pages()['source_a.test']
+        source_a_page = self.tree.get_pages()['source_a.test']
         self.assertEqual(
             source_a_page.symbol_names,
             OrderedSet(['symbol_1',
                         'symbol_2']))
 
-        doc_tree.persist()
+        self.tree.persist()
 
         # Now we'll try removing page_x altogether
         self.__remove_md_file('page_x.markdown')
@@ -605,14 +625,13 @@ class TestDocTree(unittest.TestCase):
                '\tcore_page.markdown\n')
 
         new_sitemap = self.__parse_sitemap(inp)
-        doc_tree = self.__update_test_layout(doc_tree, new_sitemap)
-        self.__assert_stale(doc_tree,
-                            set(['source_b.test']))
-        source_b_page = doc_tree.get_pages()['source_b.test']
+        self.__update_test_layout(new_sitemap)
+        self.__assert_stale(set(['source_b.test']))
+        source_b_page = self.tree.get_pages()['source_b.test']
         self.assertEqual(
             source_b_page.symbol_names,
             OrderedSet(['symbol_4', 'symbol_3']))
-        doc_tree.persist()
+        self.tree.persist()
 
         # And rollback again
         self.__create_md_file(
@@ -621,17 +640,98 @@ class TestDocTree(unittest.TestCase):
              'symbols: [symbol_3]\n'
              '...\n'
              '# Page X\n'))
-        doc_tree = self.__update_test_layout(doc_tree, sitemap)
-        self.__assert_stale(doc_tree,
-                            set(['page_x.markdown',
+        self.__update_test_layout(sitemap)
+        self.__assert_stale(set(['page_x.markdown',
                                  'source_b.test']))
 
-        page_x = doc_tree.get_pages()['page_x.markdown']
+        page_x = self.tree.get_pages()['page_x.markdown']
         self.assertEqual(page_x.symbol_names, OrderedSet(['symbol_3']))
 
-        source_b_page = doc_tree.get_pages()['source_b.test']
+        source_b_page = self.tree.get_pages()['source_b.test']
         self.assertEqual(
             source_b_page.symbol_names,
             OrderedSet(['symbol_4']))
 
-        doc_tree.persist()
+        self.tree.persist()
+
+    def test_index_override_incremental(self):
+        sitemap = self.__create_test_layout()
+        self.tree.persist()
+        index_page = self.tree.get_pages()['test-index']
+        self.assertIn('source_b.test', index_page.subpages)
+
+        self.incremental = True
+
+        self.__touch_src_file('test-index.markdown')
+        self.__update_test_layout(sitemap)
+        index_page = self.tree.get_pages()['test-index']
+        self.assertIn('source_b.test', index_page.subpages)
+        self.tree.persist()
+
+    def test_extension_index_only(self):
+        inp = (u'test-index\n'
+               '\ttest-section.markdown\n')
+        self.test_ext.setup()
+        sitemap = self.__parse_sitemap(inp)
+        self.__create_md_file(
+            'test-section.markdown',
+            u'# My test section\n')
+        self.tree.parse_sitemap(sitemap)
+        self.__assert_extension_names(
+            self.tree,
+            {u'test-index': 'test-extension',
+             u'test-section.markdown': 'test-extension'})
+
+    def test_extension_auto_sorted_override(self):
+        self.__create_md_file(
+            'source_b.test.markdown',
+            (u'---\nauto-sort: true\n...\n# My override\n'))
+        sitemap = (u'index.markdown\n'
+                   '\ttest-index\n'
+                   '\t\ttest-section.markdown\n'
+                   '\t\t\tsource_b.test\n'
+                   '\t\t\tsource_a.test\n'
+                   '\t\tpage_x.markdown\n'
+                   '\t\tpage_y.markdown\n'
+                   '\tcore_page.markdown\n')
+        _ = self.__create_test_layout(sitemap=sitemap)
+        pages = self.tree.get_pages()
+        self.assertTrue(pages['source_a.test'].pre_sorted)
+        self.assertFalse(pages['source_b.test'].pre_sorted)
+
+        self.__create_md_file(
+            'source_b.test.markdown',
+            (u'# My override\n'))
+        sitemap = (u'index.markdown\n'
+                   '\ttest-index\n'
+                   '\t\ttest-section.markdown\n'
+                   '\t\t\tsource_b.test\n'
+                   '\t\t\tsource_a.test\n'
+                   '\t\tpage_x.markdown\n'
+                   '\t\tpage_y.markdown\n'
+                   '\tcore_page.markdown\n')
+        _ = self.__create_test_layout(sitemap=sitemap)
+        pages = self.tree.get_pages()
+        self.assertTrue(pages['source_a.test'].pre_sorted)
+        self.assertTrue(pages['source_b.test'].pre_sorted)
+
+    def test_extension_implicit_override(self):
+        self.__create_md_file(
+            'source_b.test.markdown',
+            (u'---\nsymbols:\n  - symbol_2\n...\n# My override\n'))
+        _ = self.__create_test_layout()
+
+        source_b = self.tree.get_pages()['source_b.test']
+        self.assertEqual(
+            os.path.basename(source_b.source_file),
+            'source_b.test.markdown')
+        self.assertEqual(source_b.symbol_names, ['symbol_2', 'symbol_4'])
+
+        source_a = self.tree.get_pages()['source_a.test']
+        self.assertEqual(source_a.symbol_names, ['symbol_1'])
+
+        out, _ = cmark.ast_to_html(source_b.ast, None)
+
+        self.assertEqual(
+            out,
+            u'<h1>My override</h1>\n')
